// See DTPlotView.h

#import "DTPlotView.h"

#import "DTMatlabDataFile.h"
#import "DTPlot1D.h"
#import "DTRegion1D.h"
#import "DTFloatArray.h"

#import <Cocoa/Cocoa.h>

@implementation DTPlotView

- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil) {
		// Add initialization code here
		
		entryName = [@"Plot" retain]; // Try also Plot2
		fileName = [@"../Data Files/Variables.mat" retain];
	}
	return self;
}

- (void)drawRect:(NSRect)rect
{
	NSRect myFrame = [self frame];
	
	// Draw a white background
	[[NSColor whiteColor] set];
	myFrame.origin = NSMakePoint(0,0);
	if ([[NSGraphicsContext currentContext] isDrawingToScreen]) {
		// Otherwise it's creating a tiff or pdf image.
		NSRectFill(myFrame);	
	}
	
	if (fileName==nil || entryName==nil) return;
	if ([[NSFileManager defaultManager] fileExistsAtPath:fileName]==NO) return;
	
	// Open the data file
	DTMatlabDataFile inputFile([fileName fileSystemRepresentation],DTFile::ReadOnly);
	
	// When running in the debugger, you can see the content of the file with
	// call inputFile.pinfo()
	
	// Read in the plot.  Use call thePlot.pinfo() to get information about it in the debugger.
	DTPlot1D thePlot;
	ReadFromFile(inputFile,string([entryName cString]),thePlot);
	if (thePlot.IsEmpty()) return;
	
	DTRegion1D xRange = BoundingRange(thePlot);
	DTRegion1D yRange = ValueRange(thePlot);
	// Can overwrite this with xRange = DTRegion1D(minValue,maxValue);
	
	// The portion to draw the plot into
	NSRect drawingPortion;
	drawingPortion.origin = NSMakePoint(70,30);
	drawingPortion.size.width = myFrame.size.width - drawingPortion.origin.x - 20;
	drawingPortion.size.height = myFrame.size.height - drawingPortion.origin.y - 20;
	
	// The scale to go from the range into drawingPortion.size
	// x -> shiftX+scaleX*x
	double scaleX = drawingPortion.size.width/(xRange.maxV-xRange.minV);
	// Want shiftX+scaleX*xRange.minV = drawingPortion.origin.x
	double shiftX = drawingPortion.origin.x - scaleX*xRange.minV;
	double scaleY = drawingPortion.size.height/(yRange.maxV-yRange.minV);
	double shiftY = drawingPortion.origin.y - scaleY*yRange.minV;
	
	// The Quartz routines want to get the Graphics Context.
	CGContextRef ctx = (CGContextRef) [[NSGraphicsContext currentContext] graphicsPort];
	
	// Draw the plot.
	DTDoubleArray Data = thePlot.Data();
	DTMutableFloatArray singleLoop;

	// The color of the plot
	[[NSColor blackColor] set];
	// Line width
	CGContextSetLineWidth(ctx,1.0);

	// Draw the path as a collection of loops.
	CGContextBeginPath(ctx);
	int len = Data.n();
	int i,until,pos,howMany;
	int loc = 0;
	while (loc<len) {
		howMany = int(Data(1,loc));
		until = loc+howMany+1;
		singleLoop = DTMutableFloatArray(2,howMany);
		
		pos = 0;
		for (i=loc+1;i<until;i++) {
			singleLoop(0,pos) = shiftX + scaleX*Data(0,i);
			singleLoop(1,pos) = shiftY + scaleY*Data(1,i);
			// Skip Inf and NaN's.
			if (finite(singleLoop(0,pos)) && finite(singleLoop(1,pos)))
				pos++;
		}
		if (pos>1) CGContextAddLines(ctx,(CGPoint *)singleLoop.Pointer(),pos);
		
		loc = until;
	}
	CGContextStrokePath(ctx);
	
	// Draw the box around the plot
	NSFrameRect(drawingPortion);

	// Figure out the step sizes.
	double stepX = 1;
	while (stepX*scaleX<10) stepX*=10.0;
	while (stepX*scaleX>100) stepX/=10.0;
	if (stepX*scaleX>70) stepX /= 2;
	else if (stepX*scaleX<20) stepX *= 5;
	else if (stepX*scaleX<40) stepX *= 2;
	
	double stepY = 1;
	while (stepY*scaleY<10) stepY*=10.0;
	while (stepY*scaleY>100) stepY/=10.0;
	if (stepY*scaleY>70) stepY /= 2;
	else if (stepY*scaleY<20) stepY *= 5;
	else if (stepY*scaleY<30) stepY *= 2;

	
	// Draw the tick marks.
	CGContextBeginPath(ctx);
	float x,y;
	// Draw the tick marks
	for (i=int(ceil(xRange.minV/stepX-1e-4));i<=int(floor(xRange.maxV/stepX+1e-4));i++) {
		x = shiftX + scaleX*i*stepX;
		CGContextMoveToPoint(ctx,x,drawingPortion.origin.y);
		CGContextAddLineToPoint(ctx,x,drawingPortion.origin.y-6.0);
	}
	for (i=int(ceil(yRange.minV/stepY-1e-4));i<=int(floor(yRange.maxV/stepY+1e-4));i++) {
		y = shiftY + scaleY*i*stepY;
		CGContextMoveToPoint(ctx,drawingPortion.origin.x,y);
		CGContextAddLineToPoint(ctx,drawingPortion.origin.x-6.0,y);
	}
	CGContextStrokePath(ctx);
	
	// Now the labels.
	// Font information is stored in an attribute dictionary.
	NSMutableDictionary *theAttributes = [NSMutableDictionary dictionaryWithCapacity:3];
	[theAttributes setObject:[NSFont fontWithName:@"Helvetica" size:12.0]
					  forKey:NSFontAttributeName];
	[theAttributes setObject:[NSColor blackColor]
					  forKey:NSForegroundColorAttributeName];
	

	NSString *label;
	NSSize theSize;
	for (i=int(ceil(xRange.minV/stepX-1e-4));i<=int(floor(xRange.maxV/stepX+1e-4));i++) {
		x = shiftX + scaleX*i*stepX;
		y = drawingPortion.origin.y - 7.0;

		label = [NSString stringWithFormat:@"%g",i*stepX];
		theSize = [label sizeWithAttributes:theAttributes];
		[label drawAtPoint:NSMakePoint(x-theSize.width/2.0,y-theSize.height) withAttributes:theAttributes];
	}
	for (i=int(ceil(yRange.minV/stepY-1e-4));i<=int(floor(yRange.maxV/stepY+1e-4));i++) {
		x = drawingPortion.origin.x - 9.0;
		y = shiftY + scaleY*i*stepY;

		label = [NSString stringWithFormat:@"%g",i*stepY];
		theSize = [label sizeWithAttributes:theAttributes];
		[label drawAtPoint:NSMakePoint(x-theSize.width,y-theSize.height/2.0) withAttributes:theAttributes];
	}
	
	// X title and plot title are done the same way.  The rotated y title is slightly trickier
	CGAffineTransform trans,shift;
	double rotation = M_PI/2.0;
	NSPoint rotateAround = NSMakePoint(15.0,drawingPortion.origin.y+drawingPortion.size.height/2.0);
	// Save the state.  Pushes the current state onto a stack (first in, last out)
	CGContextSaveGState(ctx);
	shift = CGAffineTransformMakeTranslation(-rotateAround.x,-rotateAround.y); // Shift point to origin
	trans = CGAffineTransformMakeRotation(rotation); // Rotate around origin
	trans = CGAffineTransformConcat(shift,trans); // Combine the above to transformations
	shift = CGAffineTransformMakeTranslation(rotateAround.x,rotateAround.y); // translate origin back
	trans = CGAffineTransformConcat(trans,shift); // Add this transformation
	CGContextConcatCTM(ctx,trans); // Apply transformation to drawing context
	
	// Now draw the string at the specified point.
	label = @"This is the y title";
	theSize = [label sizeWithAttributes:theAttributes];
	[label drawAtPoint:NSMakePoint(rotateAround.x-theSize.width/2.0,rotateAround.y-theSize.height/2.0)
		withAttributes:theAttributes];
	
	// Restore the graphics state
	CGContextRestoreGState(ctx);
}

- (IBAction)setEntryName:(id)sender
{
	[entryName release];
	entryName = [[sender stringValue] retain];
	
	[self setNeedsDisplay:YES];
}

- (IBAction)setFileName:(id)sender
{
	[fileName release];
	fileName = [[sender stringValue] retain];

	[self setNeedsDisplay:YES];
}

- (void)copyPDF:(id)sender
{
	NSRect myFrame = [self frame];
	myFrame.origin = NSMakePoint(0,0);
	NSData *pdfData = [self dataWithPDFInsideRect:myFrame];
	
	NSPasteboard *thePasteboard = [NSPasteboard generalPasteboard];
	[thePasteboard declareTypes:[NSArray arrayWithObject:NSPDFPboardType] owner:nil];
	[thePasteboard setData:pdfData forType:NSPDFPboardType];

	// If you want to save this to the file /tmp/image.pdf instead of the clipboard, do
	// [pdfData writeToFile:@"/tmp/image.pdf" atomically:NO];
}

- (void)copyTIFF:(id)sender
{
	NSRect myFrame = [self frame];
	
	NSImage *theImage = [[NSImage alloc] initWithSize:myFrame.size];
	[theImage lockFocus];
	
	[self drawRect:myFrame]; // Done for convenience.  You don't need to do this inside a NSView object.

	[theImage unlockFocus];
	
	NSData *tiffData = [theImage TIFFRepresentationUsingCompression:NSTIFFCompressionLZW factor:0.0];
	NSPasteboard *thePasteboard = [NSPasteboard generalPasteboard];
	[thePasteboard declareTypes:[NSArray arrayWithObject:NSTIFFPboardType] owner:nil];
	[thePasteboard setData:tiffData forType:NSTIFFPboardType];
	
	// If you want to save this to the file /tmp/image.tiff instead of the clipboard, do
	// [tiffData writeToFile:@"/tmp/image.tiff" atomically:NO];
}

@end
