#include "DTSource.h"
#include "DTSaveError.h"

#include "DTDataContainer.h"

// Structures needed to support the return group

struct DT_RetGroup {
    DTMesh2D output;
    DTPlot1D row;
};

extern void WriteToFile(DTDataStorage &output,string name,const DT_RetGroup &var);

void WriteToFile(DTDataStorage &output,string name,const DT_RetGroup &var)
{
    // Write the data.
    WriteToFile(output,name+"_output",var.output);
    WriteToFile(output,name+"_row",var.row);
    
    // Write the structure information.
    output.Save("output",name+"_0_name");
    output.Save("Mesh2D",name+"_0_type");
    output.Save("row",name+"_1_name");
    output.Save("Plot1D",name+"_1_type");
    
    // To be self documenting, need to specify group count.
    output.Save(2,name);
}

// End of group structure. 


DT_RetGroup Computation(DTFile fileIn,double rowNumber,string name);

int main(int argc,const char *argv[])
{
    DTSetArguments(argc,argv);
    
    DTDataContainer ioStream;
    DTMutablePointer<DTDataStorage> inputSource;
    
    // Get data either from DataTank or a data file.
    bool socketCommunication = DTArgumentIncludesFlag("socket");
    if (socketCommunication) {
        if (!ioStream.ConnectToServerSocket(DTArgumentFlagEntry("socket")))
            return 0;
        if (!ioStream.ReadFromSocket()) return 0;
        inputSource = ioStream.AsPointer();
    }
    else {
        DTMatlabDataFile inputFile("Input.mat",DTFile::ReadOnly);
        inputSource = inputFile.AsPointer();
    }
    
    // Read in the input variables.
    DTFile fileIn("fileIn",DTFile::ReadOnly);
    double rowNumber = inputSource->ReadNumber("rowNumber");
    string name = inputSource->ReadString("name");
    
    if (socketCommunication) ioStream.RemoveAllEntries();
    
    // The computation.
    DT_RetGroup computed;
    clock_t t_before = clock();
    computed = Computation(fileIn,rowNumber,name);
    clock_t t_after = clock();
    double exec_time = double(t_after-t_before)/double(CLOCKS_PER_SEC);
    
    // Write the output.
    // Send the output either to DataTank through a socket or a data file.
    DTMutablePointer<DTDataStorage> outputContainer;
    if (socketCommunication) {
        outputContainer = inputSource;
    }
    else {
        DTMatlabDataFile outputFile("Output.mat",DTFile::NewReadWrite);
        outputContainer = outputFile.AsPointer();
    }
    
    WriteToFile(*outputContainer,"Var",computed);
    outputContainer->Save("Group","Seq_Var");
    
    // The execution time.
    outputContainer->Save(exec_time,"ExecutionTime");
    outputContainer->Save("Real Number","Seq_ExecutionTime");
    
    // The errors.
    DTSaveError(*outputContainer,"ExecutionErrors");
    outputContainer->Save("StringList","Seq_ExecutionErrors");
    
    if (socketCommunication) {
    	// Transfer the data over.
    	ioStream.WriteIntoSocket();
    	ioStream.TerminateCommunication();
    }
    
    return 0;
}

DT_RetGroup Computation(DTFile fileIn,double rowNumber,string name)
{
    DT_RetGroup toReturn;

    DTMatlabDataFile theFile(fileIn);

    // Read in the mesh.  It will be empty if the variable doesn't exist.
    // You can catch the error message.  You can also use the Contains(...)
    // variable to see if the variable exists.
    ReadFromFile(theFile,name,toReturn.output);
    
    // Now extract a single row
    DTMesh2D theMesh = ConvertToDouble(toReturn.output);  // In case you send in a single precision array.
    DTDoubleArray data = theMesh.DoubleData();
    
    // Round to the nearest grid point.
    int i =  int(floor((rowNumber-theMesh.Grid().Origin().x)/theMesh.Grid().dx()+0.5));
    if (i<0) i = 0;
    if (i>data.m()) i = data.m()-1;
    
    // Now extract a single row
    DTDoubleArray singleRow = data(i,DTIndex::All);
    
    // Create the plot array.  Need to adhere to the plot format.
    DTMutableDoubleArray plotArray(2,singleRow.Length()+1);
    plotArray(0,0) = 0;
    plotArray(1,0) = singleRow.Length();
    plotArray(1,DTIndex(1,DTIndex::Last)) = singleRow;
    
    double h = theMesh.Grid().dy();
    double origin = theMesh.Grid().Origin().y;
    int n = singleRow.Length();
    for (i=0;i<n;i++) {
        plotArray(0,i+1) = i*h+origin;
    }

    toReturn.row = DTPlot1D(plotArray);

    return toReturn;
}
